/*
    fdcc_conf_2008: logiciel DOS permetant de configurer et piloter la centrale free-dcc-2008

    Ulysse Delmas-Begue    25 fev 2008 : adaptation a free-dcc-2008 a partir de fdcc_conf
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#define STRICT
#include <windows.h>
//#include <commdlg.h>
//#PRAGMA "CommDlg32.lib"





/*****************************************************************************/
/* LIAISON SERIE                                                             */
/*****************************************************************************/

/*------------------------------------------------------------------------------
NAME  : cSerial

DESCR : class cSerial: Permet de communiquer avec la carte.

NOTES :
------------------------------------------------------------------------------*/
class cSerial
{
    public:

    DCB     dcb;          // description de la liaison serie
    HANDLE  hCom;         // handler sur la liaison derie
    COMSTAT Status;       // pour voir si rx ok

         cSerial(void);                // constructeur (ne fait rien)
    int  init(char* name,int baud);    // ouvre le port et l'initialise
    int  close(void);                  // ferme le port
    int  read(int* data);              // effectue une lecture securisee sue la rs232
    int  blocking_read_timeout_100ms(int* data); // effectue une lecture securisee sue la rs232 (attend la date pendant 100ms max)
    int  read_during_5s(int* data);    // lecture bloquante pendant 5s
    int  write(int data);              // effectue une ecriture
    void dtr(int state);               // permet de mettre un etat sur la ligne DTR
    void rts(int state);               // permet de mettre un etat sur la ligne RTS
};


/*------------------------------------------------------------------------------
NAME  : cSerial::cSerial
DESCR : constructeur de la classe serial.
        (ne fait rien)
ARG   : void
RETURN: void
NOTES : none
------------------------------------------------------------------------------*/
cSerial::cSerial(void)
{
}


/*------------------------------------------------------------------------------
NAME  : init
DESCR : ouvre le port serie et l'initialise
ARG   : char* name: nom du port serie (COM1,COM2...)
        int   vitesse: (9600,19200...)
RETURN: 1 OK or 0 ERROR
NOTES : l'OS ferme automatiquement les ports lorsque l'application se ermine
------------------------------------------------------------------------------*/
int cSerial::init(char* name, int baud)
{
    hCom=CreateFile(name,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
    if(hCom==INVALID_HANDLE_VALUE) { printf("Impossible d'ouvrir le port serie (CreateFile)");   return 0; }
    if(!GetCommState(hCom,&dcb))   { printf("Impossible d'ouvrir le port serie (GetCommState)"); return 0; }
    memset(&dcb,0,sizeof(dcb));
    if(     baud==1200)  dcb.BaudRate = CBR_1200;
    else if(baud==2400)  dcb.BaudRate = CBR_2400;
    else if(baud==4800)  dcb.BaudRate = CBR_4800;
    else if(baud==9600)  dcb.BaudRate = CBR_9600;
    else if(baud==19200) dcb.BaudRate = CBR_19200;
    else                           { printf("La vitesse specifiee n'est pas suportee");          return 0; }
    dcb.ByteSize = 8;
    dcb.Parity   = NOPARITY;
  //dcb.StopBits = ONESTOPBIT;
    dcb.StopBits = TWOSTOPBITS;
    if(!SetCommState(hCom,&dcb))   { printf("Impossible d'ouvrir le port serie (SetCommState)"); return 0;}
    return 1;
}


/*------------------------------------------------------------------------------
NAME  : close
DESCR : ferme le port serie
ARG   : void
RETURN: 1 OK
NOTES : l'OS ferme automatiquement les ports lorsque l'application se termine
------------------------------------------------------------------------------*/
int cSerial::close(void)
{
    //fclose(hCom); //pose PB de compilation si on le decommente: comprendre pourquoi
    if (CloseHandle(hCom) == 0) return 0;
    return 1;
}


/*------------------------------------------------------------------------------
NAME  : cSerial::read
DESCR : effectue une lecture securise d'un octet sur la rs232
ARG   : int* data pointeur sur la donnee lue
RETURN: 1 ok or 0 nothing
NOTES : none
------------------------------------------------------------------------------*/
int cSerial::read(int* data)
{
    *data=0;
    DWORD num;
    DWORD buf[1];
    DWORD No_Error;

    ClearCommError(hCom,&No_Error,&Status);
    if(Status.cbInQue==0) { printf("####"); return 0; }
    ReadFile(hCom,buf,1,&num,NULL);
    *data=(int)(buf[0]&0xff);
    return 1;
}

/*------------------------------------------------------------------------------
NAME  : cSerial::read_during_5s
DESCR : effectue une lecture securise d'un octet sur la rs232 en attendant durant le timeout
ARG   : int* data pointeur sur la donnee lue
RETURN: 1 ok or 0 nothing
NOTES : none
------------------------------------------------------------------------------*/
int cSerial::read_during_5s(int* data)
{
    *data=0;
    int i;
    DWORD num;
    DWORD buf[1];
    DWORD No_Error;

    for(i=0;i<500;i++)
    {
        ClearCommError(hCom,&No_Error,&Status);
        //if(Status.cbInQue>=2) {printf(">>>>%x",No_Error); Sleep(2000); }
        if(Status.cbInQue!=0) break;
        Sleep(10);
        if(i==4500); // { Sleep(1000); write(0); Sleep(1000); }
    }
    if(Status.cbInQue==0) { /*printf("####"); printf("%x####",No_Error); Sleep(2000);*/ return 0; }
    //for(i=0;i<Status.cbInQue;i++) ReadFile(hCom,buf,1,&num,NULL);
    ReadFile(hCom,buf,1,&num,NULL);
    *data=(int)(buf[0]&0xff);
    return 1;
}

/*------------------------------------------------------------------------------
NAME  : cSerial::blocking_read_timeout_100ms
DESCR : effectue une lecture securise d'un octet sur la rs232 (attend la date pendant 100ms max)
ARG   : int* data pointeur sur la donnee lue
RETURN: 1 ok or 0 nothing
NOTES : none
------------------------------------------------------------------------------*/
int cSerial::blocking_read_timeout_100ms(int* data)
{
    *data=0;
    DWORD num;
    DWORD buf[1];
    DWORD No_Error;
    int i;

    i=0;
    do
    {
        ClearCommError(hCom,&No_Error,&Status);
        if(Status.cbInQue==0) {Sleep(1); i++; if(i==100) return 0;}
    }
    while(Status.cbInQue==0);

    ReadFile(hCom,buf,1,&num,NULL);
    *data=(int)buf[0];
    *data=(*data)&0xff;
    return 1;
}

/*------------------------------------------------------------------------------
NAME  : cSerial::write
DESCR : ecrit 1 octet
ARG   : void
RETURN: 1 ok
NOTES : none
------------------------------------------------------------------------------*/
int cSerial::write(int data)
{
    DWORD num;
    DWORD buf[1];

    buf[0]=(DWORD)(data&0xff);
    WriteFile(hCom,buf,1,&num,NULL);
    return 1;
}


/*------------------------------------------------------------------------------
NAME  : cSerial::dtr
DESCR : permet de mettre un etat sur la ligne DTR
ARG   : int state: etat de la ligne DTR: 0 pour -12V et 1 pour +12V
RETURN: void
NOTES : none
------------------------------------------------------------------------------*/
void cSerial::dtr(int state)
{
    if(state==0) EscapeCommFunction(hCom,CLRDTR);
    else         EscapeCommFunction(hCom,SETDTR);
}


/*------------------------------------------------------------------------------
NAME  : cSerial::rts
DESCR : ermet de mettre un etat sur la ligne RTS
ARG   : int state: etat de la ligne RTS: 0 pour -12V et 1 pour +12V
RETURN: void
NOTES : none
------------------------------------------------------------------------------*/
void cSerial::rts(int state)
{
    if(state==0) EscapeCommFunction(hCom,CLRRTS);
    else         EscapeCommFunction(hCom,SETRTS);
}





cSerial   serial;
int icom=2;
char sz_com[100]="COM2";



void select_port(void)
{
    printf("\nNumero de port --> ");
    scanf(" %d",&icom);
    sprintf(sz_com,"\\\\.\\COM%d",icom);
}



void test_link(void)
{
    int i,data;

    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    if(data==0x83) printf("\nLa carte repond bien, communication OK");
    else           printf("\nProbleme de communication avec la carte");
    serial.close();

    Sleep(3000);
}



/*
Protocole UU:
;=============================================================================
;  Moniteur RS232 (19200bauds)
;    - dure env 40us
;    - a appeller au minimum toutes les 4ms
;
;  0: sync
;  1: test         ->0x83
;  2: setadr            + adr
;  3: write             + dat
;  4: read         ->dat 
;  5: inc+write         + dat
;  6: inc+read     ->dat
;  7: inc+read+clr ->dat 
;  DESACTIVE: 8: read16     ->dat (precedement sauve lors du dernier read (a l'adresse suivante))
;  8: mode              + dat (bit0=IRP)
;  9: transfer de masse 72x + dat -> dat (voir specificites dans rs_cmd_14)
;
;  attention: au niveau du PC:
;  - a 19200 bps, rajouter 1ms entre l'envoie de 2 packets, 
;   sinon UART micro plante et micro ecrit n'importe ou dont 
;   en rs_config ce qui peut provoquer une ecriture flash ds le programme !
;   c'est pourquoi les ecritures sont maintenant protegees !
;   pas de PB avec les vitesses plus lentes
; - entre une emission et reception sur mon PC, j'ai note un besoin 
;   de 4ms a 19200bps et 8ms a 9600bps
;
;=============================================================================
*/



void uu_irf(int val)
{
    serial.write(8); Sleep(10); serial.write(val); Sleep(10);
}



void uu_write(int adr,int dat)
{
    serial.write(2); Sleep(10); serial.write(adr); Sleep(10); // set ADR
    serial.write(3); Sleep(10); serial.write(dat); Sleep(10); // write
}


int pb_cpt=0;
int uu_read(int adr)
{
    int dat,ret,dat1,ret1,dat2,ret2;
    char c;

    //serial.write(2); Sleep(10); serial.write(adr); Sleep(10); // set ADR
    //serial.write(4); Sleep(10); serial.read(&dat); Sleep(10); // read

    //serial.write(2); Sleep(10); serial.write(adr);           Sleep(10); // set ADR
    //serial.write(4); Sleep(10); serial.read_during_5s(&dat); Sleep(10); // read

    serial.write(2);
    serial.write(adr);
    serial.write(4);
    ret=serial.read_during_5s(&dat);
    if(ret==0)
    {
        pb_cpt++;
        printf("\n--> Stall detected");
        serial.write(1);
        ret=serial.read_during_5s(&dat);
        ret2=serial.read_during_5s(&dat2);
        printf(" ret1=%d dat1=%d ret2=%d dat2=%d pb_cpt=%d <--\n",ret,dat,ret2,dat2,pb_cpt);
        Sleep(1000);
        if(ret2==0 || dat2!=0x83) { printf("\n\nPanic\n\n"); scanf(" %c",&c); exit(1); }
    }

    return dat;
}





int uu_read_clr(int adr)
{
    int dat;

//    serial.write(2); Sleep(10); serial.write(adr); Sleep(10); // set ADR
//  //serial.write(7); Sleep(10); serial.read(&dat); Sleep(10); // read
//    serial.write(4); Sleep(10); serial.read(&dat); Sleep(10); // read
      dat=uu_read(adr);
      serial.write(3); Sleep(10); serial.write(0);   Sleep(10); // write

    return dat;
}










/*****************************************************************************/
/* DEBUG                                                                     */
/*****************************************************************************/


// n'utilise plus les transferts de masse
void main_disp_inputs(void)
{
    int i,j,data,k,nb;
    int rs232_in_mem[8];
    int rs232_in_rd[8];

    serial.init(sz_com,19200);
    printf("Veuillez patienter (vidange de la liaison serie)");
    for(i=0;i<100;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<100 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(4); serial.read(&data);  /*Sleep(10);*/
    printf("\rPING:%x                                            ",data); // ping

    printf("\nNB de lectures -> "); scanf(" %d",&nb);

    for(i=0;i<8;i++) rs232_in_mem[i]=0;
    //txrx_masse();
    uu_irf(1); for(i=0;i<8;i++) rs232_in_rd[i]=uu_read_clr(0x80+0x18+i); uu_irf(0);  //1ere lecture pour effacer la memorisation

    for(k=0;k<nb;k++)
    {
         //txrx_masse();
         uu_irf(1); for(i=0;i<8;i++) rs232_in_rd[i]=uu_read_clr(0x80+0x18+i); uu_irf(0);
         system("cls");
         printf("\n* entree a 1 / - entree a 0 / M memorisation de 1\n");
         printf("\n    00000000 00111111 11112222 22222233 33333333 44444444 44555555 55556666");
         printf("\n    01234567 89012345 67890123 45678901 23456789 01234567 89012345 67890123");
         printf("\n\nIN: ");

         for(j=0;j<8;j++) for(i=0;i<8;i++)
         {
            if(i==0) rs232_in_mem[j]|=rs232_in_rd[j];
            if(rs232_in_rd[j]&(1<<i)) printf("*"); else printf("-");
            if(i==7) printf(" ");
         }

         printf("\nM+: ");
         for(j=0;j<8;j++) for(i=0;i<8;i++)
         {
            if(rs232_in_mem[j]&(1<<i)) printf("M"); else printf(" ");
            if(i==7) printf(" ");
         }

         printf("\n\n%d/%d",k+1,nb);
    }

    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    serial.close();
}



void main_disp_canaux(void)
{
    int i,j,data,adr[16],vit[16],fct[16],um[4],nb;

    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    printf("\nNB de lectures --> "); scanf(" %d",&nb);

    for(j=0;j<nb;j++)
    {
        for(i=0;i<16;i++)
        {
            serial.write(8); Sleep(10); serial.write(0); Sleep(10); // clr IRF

            //serial.write(2); Sleep(5); serial.write(0x40+i); Sleep(5);  // set ADR
            //serial.write(4); Sleep(10); serial.read(&adr[i]); Sleep(10); // read
            adr[i]=uu_read(0x40+i);
            //serial.write(2); Sleep(5); serial.write(0x50+i); Sleep(5);  // set ADR
            //serial.write(4); Sleep(10); serial.read(&vit[i]); Sleep(10); // read
            vit[i]=uu_read(0x50+i);
            //serial.write(2); Sleep(5); serial.write(0x60+i); Sleep(5);  // set ADR
            //serial.write(4); Sleep(10); serial.read(&fct[i]); Sleep(10); // read
            fct[i]=uu_read(0x60+i);

            if(i>=8 && i<=11)
            {
                serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF
                //serial.write(2); Sleep(5); serial.write(0x10+i); Sleep(5);  // set ADR
                //serial.write(4); Sleep(10); serial.read(&um[i-8]); Sleep(10); // read
                um[i-8]=uu_read(0x10+i);
            }
        }

        system("cls");
        for(i=0;i<16;i++)
        {
            printf("\nCH%02d ADR=%02d VIT=0x%02X FCT=0x%02X",i,adr[i],vit[i],fct[i]);
            printf(" VIT="); if(vit[i]&1<<5) printf("+"); else printf("-");
            printf("%02d",2*(vit[i]&15)+(vit[i]&16)/16);
            printf(" FCT=");
            if(fct[i]&0x10) printf("*"); else printf("-");
            if(fct[i]&0x08) printf("4"); else printf("-");
            if(fct[i]&0x04) printf("3"); else printf("-");
            if(fct[i]&0x02) printf("2"); else printf("-");
            if(fct[i]&0x01) printf("1"); else printf("-");

            if(i>=8 && i<=11) printf(" UM_MST=%02d",um[i-8]);
        }
        printf("\n%d/%d",j+1,nb);
    }
    serial.write(8); Sleep(10); serial.write(0); Sleep(10); // clr IRF

    serial.close();
}


void modify_aig_out_led(void)
{
    int i,j,data,choix,choix0,nb,nbi;
    int aig_cmd[4];
    int aig_pos0[4];
    int aig_pos1[4];
    int out[4];
    int led[8];
    int cli[8];
    int phase[8];

    serial.init(sz_com,19200);
    for(i=0;i<100;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<10 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(4); serial.read(&data);  /*Sleep(10);*/ printf("\nPING:%x",data); // ping

    nbi=nb=0;
    while(1)
    {
        uu_irf(0);  for(i=0;i<4;i++) aig_cmd[i]=uu_read(0x80+0x30+i);
                    for(i=0;i<4;i++) aig_pos0[i]=uu_read(0x80+0x34+i);
                    for(i=0;i<4;i++) aig_pos1[i]=uu_read(0x80+0x38+i);
                    for(i=0;i<4;i++) out[i]=uu_read(0x80+0x58+i);
                    for(i=0;i<8;i++) led[i]=uu_read(0x80+0x40+i);
                    for(i=0;i<8;i++) cli[i]=uu_read(0x80+0x48+i);
        uu_irf(1);  for(i=0;i<8;i++) phase[i]=uu_read(0x50+i);
        uu_irf(0);

        system("cls");
        printf("Affichage / modification des aiguillages / sorties / leds (on/cli/phase)\n");

        printf("\n      00000000 00111111 11112222 22222233");
        printf("\n      01234567 89012345 67890123 45678901");
        printf("\n");
        printf("\ncmd:  ");
        for(j=0;j<4;j++) for(i=0;i<8;i++)
        { if(aig_cmd[j]&(1<<i)) printf("/"); else printf("-"); if(i==7) printf(" "); }
        printf("\npos0: ");
        for(j=0;j<4;j++) for(i=0;i<8;i++)
        { if(aig_pos0[j]&(1<<i)) printf("*"); else printf("-"); if(i==7) printf(" "); }
        printf("\npos1: ");
        for(j=0;j<4;j++) for(i=0;i<8;i++)
        { if(aig_pos1[j]&(1<<i)) printf("*"); else printf("-"); if(i==7) printf(" "); }
        printf("\n");
        printf("\n      00000000 00111111 11112222 22222233");
        printf("\n      01234567 89012345 67890123 45678901");
        printf("\n");
        printf("\nout:  ");
        for(j=0;j<4;j++) for(i=0;i<8;i++)
        { if(out[j]&(1<<i)) printf("*"); else printf("-"); if(i==7) printf(" "); }
        printf("\n");
        printf("\n      00000000 00111111 11112222 22222233 33333333 44444444 44555555 55556666");
        printf("\n      01234567 89012345 67890123 45678901 23456789 01234567 89012345 67890123");
        printf("\n");
        printf("\nled:  ");
        for(j=0;j<8;j++) for(i=0;i<8;i++)
        { if(led[j]&(1<<i)) printf("*"); else printf("-"); if(i==7) printf(" "); }
        printf("\ncli:  ");
        for(j=0;j<8;j++) for(i=0;i<8;i++)
        { if(cli[j]&(1<<i)) printf("*"); else printf("-"); if(i==7) printf(" "); }
        printf("\npha:  ");
        for(j=0;j<8;j++) for(i=0;i<8;i++)
        { if(phase[j]&(1<<i)) printf("*"); else printf("-"); if(i==7) printf(" "); }
        printf("\n");
        if(nbi!=nb)
        {
            printf("%d/%d",nbi+1,nb);
            nbi++;
            continue;
        }
        printf("\nchoix xyy (yy=numero x=0=aig_cmd x=1=out x=2=led x=3=cli x=4=phase)");
        printf("\n (fin-1) (lect-2) (scan-3) --> ");
        scanf(" %d",&choix);

        if(choix==-3)
        {
            printf("\nnombre de scan -> "); scanf(" %d",&nb);
            nbi=0;
        }

        if(choix==-1) break;

        if(choix>=0 && choix<=31)
        {
            choix0=choix;
            aig_cmd[choix0/8]^=1<<(choix0%8);
            uu_write(0x80+0x30+choix0/8,aig_cmd[choix0/8]);
        }

        if(choix>=100 && choix<=131)
        {
            choix0=choix-100;
            out[choix0/8]^=1<<(choix0%8);
            uu_write(0x80+0x58+choix0/8,out[choix0/8]);
        }

        if(choix>=200 && choix<=263)
        {
            choix0=choix-200;
            led[choix0/8]^=1<<(choix0%8);
            uu_write(0x80+0x40+choix0/8,led[choix0/8]);
        }

        if(choix>=300 && choix<=363)
        {
            choix0=choix-300;
            cli[choix0/8]^=1<<(choix0%8);
            uu_write(0x80+0x48+choix0/8,cli[choix0/8]);
        }

        if(choix>=400 && choix<=463)
        {
            choix0=choix-400;
            phase[choix0/8]^=1<<(choix0%8);
            uu_irf(1);
            uu_write(0x50+choix0/8,phase[choix0/8]);
            uu_irf(0);
        }
    }

    serial.close();
}



void test_mouse(void)
{
    int i,j,data,nb,dcpt;
    int mouse[8];
    char c;

    serial.init(sz_com,19200);
    for(i=0;i<100;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<10 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(4); serial.read(&data);  /*Sleep(10);*/ printf("\nPING:%x",data); // ping

    while(1)
    {
        printf("\nnb de scan (-1=fin) -> "); scanf(" %d",&nb);

        if(nb==-1) break;

        for(i=0;i<nb;i++)
        {
            // ADR_MOUSE_IT   EQU     0x30   ;---3 8octets PNstable_PNold_DCPTstable    PNPNDDDD
            // ADR_MOUSE_TDF  EQU     0x38   ;---3 8octets PNstableold_CPTedge          PNCCCCCC

            uu_irf(1);
            for(j=0;j<8;j++) mouse[j]=uu_read(0x80+0x38+j);
            uu_irf(0);

            system("cls");
            printf("Souris (boutons +/-/S et compteur de duree (x100ms)\n");
            for(j=0;j<8;j++)
            {
                c=' ';
                if((mouse[j]&128)==128) c='+';
                if((mouse[j]& 64)== 64) c='-';
                if((mouse[j]&192)==192) c='S';
                dcpt=mouse[j]&0x3f;
                printf("\nSouris %d : bouton=%c   cpt=%d",j,c,dcpt);
            }
            printf("\n\n%d/%d",i+1,nb);
        }
    }

    serial.close();
}



int main_dump_ram(void)
{
    int i,j,data,data2;

    serial.init(sz_com,19200);
    for(i=0;i<100;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<10 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(4); serial.read(&data);  /*Sleep(10);*/ printf("\nPING:%x",data); // ping

    printf("\n\n====bank1234====");
    for(i=0x70;i<=127;i++)
    {
        if((i%16)==0) printf("\n%x:",i);
        //serial.write(2); Sleep(10); serial.write(+i); Sleep(10); // set ADR
        //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
        data=uu_read(i);
        printf("%02x.",data);
    }

    for(j=0;j<4;j++)
    {
        serial.write(8); Sleep(10); serial.write(j>>1); Sleep(10); // set IRF
        printf("\n\n====bank%d====",j);
        for(i=0x10;i<=0x6f;i++)
        {
            if((j==0 || j==1) && i<0x20) continue;
            if((i%16)==0) printf("\n%x:",i);
            //serial.write(2); Sleep(10); serial.write(128*j+i); Sleep(10); // set ADR
            //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            data=uu_read(128*j+i);
            printf("%02x.",data);
        }
    }

    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    serial.close();

    return 0;
}





/*****************************************************************************/
/* CONFIG                                                                    */
/*****************************************************************************/


char szLaunchDir[1000]="";
char szSaveFile[1000]="";
char szOpenFile[1000]="";

/*
HWND GetConsoleHwnd(void)
{
       #define MY_BUFSIZE 1024 // Buffer size for console window titles.
       HWND hwndFound;         // This is what is returned to the caller.
       char pszNewWindowTitle[MY_BUFSIZE]; // Contains fabricated
                                           // WindowTitle.
       char pszOldWindowTitle[MY_BUFSIZE]; // Contains original
                                           // WindowTitle.

       // Fetch current window title.
       GetConsoleTitle(pszOldWindowTitle, MY_BUFSIZE);

       // Format a "unique" NewWindowTitle.
       wsprintf(pszNewWindowTitle,"%d/%d",
                   GetTickCount(),
                   GetCurrentProcessId());

       // Change current window title.
       SetConsoleTitle(pszNewWindowTitle);

       // Ensure window title has been updated.
       Sleep(40);

       // Look for NewWindowTitle.
       hwndFound=FindWindow(NULL, pszNewWindowTitle);

       // Restore original window title.
       SetConsoleTitle(pszOldWindowTitle);

       return(hwndFound);
}
*/

BOOL SelectAutoFileOpen(void)
{
    OPENFILENAME ofn;
    BOOL status;
    HANDLE hwnd;

    hwnd=GetCurrentProcess(); 

    ZeroMemory(&ofn, sizeof(ofn));
    szOpenFile[0] = '\0';  //UU:verifier sans

    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = NULL; //GetConsoleHwnd(); //NULL; //GetHwnd(); //(HANDLE) hwnd; //NULL;  //hwnd;
    ofn.lpstrFilter = "Config Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0";
    ofn.lpstrFile = szOpenFile;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrDefExt = "txt";
    ofn.Flags = OFN_EXPLORER /*| OFN_FILEMUSTEXIST| OFN_HIDEREADONLY*/;

    //MessageBox(NULL,"test","title",MB_OK);  exit(1);
    status=GetOpenFileName(&ofn);

    SetCurrentDirectory(szLaunchDir); // pour ne pas modifier le repertoire courant

    return status;
}

BOOL SelectAutoFileSave(void)
{
    OPENFILENAME ofn;
    BOOL status;

    ZeroMemory(&ofn, sizeof(ofn));
    szSaveFile[0] = '\0';  //UU:verifier sans

    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = NULL; //GetConsoleHwnd(); //NULL; //GetCurrentProcess(); //NULL;  //hwnd;
    ofn.lpstrFilter = "Config Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0";
    ofn.lpstrFile = szSaveFile;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrDefExt = "txt";
    ofn.Flags = OFN_EXPLORER /*| OFN_FILEMUSTEXIST| OFN_HIDEREADONLY*/;

    status=GetSaveFileName(&ofn);

    SetCurrentDirectory(szLaunchDir); // pour ne pas modifier le repertoire courant

    return status;
}

/*
; Table de verite
; 
; 1er  octet bit5,4,3,2 = ordre
; 2eme octet + bit0 du 1er octet = un des 512 bits (le bit 1 est en reserve)
;
; ordre:
; 00 0000 xx = l
; 00 0001 xx = ln
; 00 0010 xx = a
; 00 0011 xx = an
; 00 0100 xx = o
; 00 0101 xx = on
; 00 0110 xx = x
; 00 0111 xx = xn
; 00 1000 xx = =
; 00 1001 xx = =s
; 00 1010 xx = =c
; 00 1011 xx = =x
; 00 1100 xx = inv
; 00 1101 xx = cpt
; 00 1110 xx = nop
; 00 1111 xx = end
;
; Ne pas oublier le end
;
; aig_cmd    0xb0 | 0.00 0.1f (32)
; aig_pos0   0xb4 | 0.20 0.3f (32)
; aig_pos1   0xb8 | 0.40 0.5f (32)
; aig_res    0xbc | 0.60 0.7f (32)
; leds_on    0xc0 | 0.80 0.bf (64)
; leds_cli   0xc8 | 0.c0 0.ff (64)
; in_edge    0xd0 | 1.00 1.3f (64)
; out        0xd8 | 1.40 1.5f (32)
; bi         0xdc | 1.60 1.6f (16)
; res        0xde | 1.70 1.7f (16)
; in_edge    0xe0 | 1.80 1.bf (64)
;            0xe8 | 1.c0 1.ff (64) //variables utilkisee pour autre choses
*/

void eq_ordre_val_copute_str(int val_h, int val_l, char* sz_str)
{
    int ordre;
    int var;
    char sz_ordre[10];

    ordre=(val_h>>2)&0x0f;
    var=val_l&0xff; if(val_h&1) var+=0x100;
    switch(ordre)
    {
        case 0:  strcpy(sz_ordre,"l  "); break;
        case 1:  strcpy(sz_ordre,"ln "); break;
        case 2:  strcpy(sz_ordre,"a  "); break;
        case 3:  strcpy(sz_ordre,"an "); break;
        case 4:  strcpy(sz_ordre,"o  "); break;
        case 5:  strcpy(sz_ordre,"on "); break;
        case 6:  strcpy(sz_ordre,"x  "); break;
        case 7:  strcpy(sz_ordre,"xn "); break;
        case 8:  strcpy(sz_ordre,"=  "); break;
        case 9:  strcpy(sz_ordre,"=s "); break;
        case 10: strcpy(sz_ordre,"=c "); break;
        case 11: strcpy(sz_ordre,"=x "); break;
        case 12: strcpy(sz_str  ,"inv"); break;
        case 13:                         break;
        case 14: strcpy(sz_str  ,"nop"); break;
        case 15: strcpy(sz_str  ,"end"); break;
    }

    if(ordre==13) sprintf(sz_str,"cpt %d",val_l);

    if(ordre<=11)
    {
        if(var>=0x0   && var<=0x1f ) sprintf(sz_str,"%s aig_cmd_%d",sz_ordre,var);
        if(var>=0x20  && var<=0x3f ) sprintf(sz_str,"%s aig_pos0_%d",sz_ordre,var-0x20);
        if(var>=0x40  && var<=0x5f ) sprintf(sz_str,"%s aig_pos1_%d",sz_ordre,var-0x40);
        if(var>=0x60  && var<=0x7f ) sprintf(sz_str,"%s aig_res_%d",sz_ordre,var-0x60);
        if(var>=0x80  && var<=0xbf ) sprintf(sz_str,"%s led_%d",sz_ordre,var-0x80);
        if(var>=0xc0  && var<=0xff ) sprintf(sz_str,"%s led_cli_%d",sz_ordre,var-0xc0);

        if(var>=0x100 && var<=0x13f) sprintf(sz_str,"%s in_edge_%d",sz_ordre,var-0x100);
        if(var>=0x140 && var<=0x15f) sprintf(sz_str,"%s out_%d",sz_ordre,var-0x140);
        if(var>=0x160 && var<=0x16f) sprintf(sz_str,"%s bi_%d",sz_ordre,var-0x160);
        if(var>=0x170 && var<=0x17f) sprintf(sz_str,"%s res_%d",sz_ordre,var-0x170);

        if(var>=0x180 && var<=0x1bf) sprintf(sz_str,"%s in_%d",sz_ordre,var-0x180);
        if(var>=0x1c0 && var<=0x1ff) sprintf(sz_str,"%s illegal",sz_ordre);
    }

}



int eq_szordre_szvar_to_nordre_nvar(char* sz_ordre, char* sz_var,int* n_ordre,int* n_var)
{
    int ordre,var;

    ordre=-1;
    if(0==stricmp(sz_ordre,"l" )) ordre=0;
    if(0==stricmp(sz_ordre,"ln")) ordre=1;
    if(0==stricmp(sz_ordre,"a" )) ordre=2;
    if(0==stricmp(sz_ordre,"an")) ordre=3;
    if(0==stricmp(sz_ordre,"o" )) ordre=4;
    if(0==stricmp(sz_ordre,"on")) ordre=5;
    if(0==stricmp(sz_ordre,"x" )) ordre=6;
    if(0==stricmp(sz_ordre,"xn")) ordre=7;
    if(0==stricmp(sz_ordre,"=" )) ordre=8;
    if(0==stricmp(sz_ordre,"=s")) ordre=9;
    if(0==stricmp(sz_ordre,"=c")) ordre=10;
    if(0==stricmp(sz_ordre,"=x")) ordre=11;
    if(0==stricmp(sz_ordre,"cpt")) ordre=13;

    var=-1;
    if(sz_var[0]>='0' && sz_var[0]<='9' ) var=atoi(sz_var);
    if(0==strnicmp(sz_var,"aig_cmd_" ,8)) var=0    +atoi(&sz_var[8]);
    if(0==strnicmp(sz_var,"aig_pos0_",9)) var=0x20 +atoi(&sz_var[9]);
    if(0==strnicmp(sz_var,"aig_pos1_",9)) var=0x40 +atoi(&sz_var[9]);
    if(0==strnicmp(sz_var,"aig_res_" ,8)) var=0x60 +atoi(&sz_var[8]);
    if(0==strnicmp(sz_var,"led_"     ,4)) var=0x80 +atoi(&sz_var[4]);
    if(0==strnicmp(sz_var,"led_cli_" ,8)) var=0xc0 +atoi(&sz_var[8]);
    if(0==strnicmp(sz_var,"in_"      ,3)) var=0x180+atoi(&sz_var[3]);
    if(0==strnicmp(sz_var,"in_edge_" ,8)) var=0x100+atoi(&sz_var[8]);
    if(0==strnicmp(sz_var,"out_"     ,4)) var=0x140+atoi(&sz_var[4]);
    if(0==strnicmp(sz_var,"bi_"      ,3)) var=0x160+atoi(&sz_var[3]);
    if(0==strnicmp(sz_var,"res_"     ,5)) var=0x170+atoi(&sz_var[5]);

    if(ordre==-1 || var==-1) { *n_ordre=-1; *n_var=-1; return 0; }
    ordre=ordre<<2; if(var>=0x100) ordre++; *n_ordre=ordre; *n_var=var&0xff; return 1;
}



/*
;  0: sync
;  1: test       ->0x83
;  2: setadr            + adr
;  3: write             + dat
;  4: read       ->dat 
;  5: inc+write         + dat
;  6: inc+read   ->dat
;  7: read+clear ->dat
;  DESACTIVE: 8: read16     ->dat (precedement sauve lors du dernier read (a l'adresse suivante))
;  8: mode              + dat (bit0=IRP)
;  9: transfer de masse 72x + dat -> dat (voir specificites dans rs_cmd_14)
*/



void read_conf(void)
{
    FILE* pf;
    int j,i,k,l,data,var,hour,min,block;
    int buff[20];
    char sz_str[100];

    // notes: lecture 4/4 sinon mauvaises lectures

    // 1. Initialisations
    printf("\n- 1 Ouverture du fichier et de la liaison serie");
    SelectAutoFileSave();
    pf=fopen(szSaveFile,"wt");
    //pf=fopen("fdcc_conf_lect.txt","wt");
    if(pf==NULL) {printf("\nErreur: Ne peut pas creer le %s\n",szSaveFile); return; }
    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(100); serial.read(&data);  Sleep(10);
    if(data!=0x83) {printf("\nErreur: Centrale non detectee (%x)\n",data); return; }
    fprintf(pf,  "# Ce fichier contient la configuraion de la centrale FREE-DCC");
    fprintf(pf,"\n#");
    fprintf(pf,"\n# - 256 CC (variable en EEPROM 0x00-0xff)");
    fprintf(pf,"\n# - 32 itineraires (table en flash 0x1500-0x15ff)");
    fprintf(pf,"\n# - 256 elements d'equations (table en flash 0x1600-0x17ff)");
    fprintf(pf,"\n#");

    // 2. Leture des CC en EEPROM
    printf("\n- 2 Leture des CC en EEPROM");
    fprintf(pf,"\n\n[cc]");
    serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2/3 (mode.0=1)
    for(j=0;j<64;j++)
    {
        // preparation de la lecture
        serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);       // adr = fl_adrh
        serial.write(3); Sleep(5); serial.write(0); Sleep(5);          // fl_adrh=0

        serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);       // adr = fl_adrl 00
        serial.write(3); Sleep(5); serial.write((4*j)&255); Sleep(5);  // fl_adrl=4*j

        serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);       // adr = fl_nb
        serial.write(3); Sleep(5); serial.write(4); Sleep(5);          // fl_nb = 4

        // ordre de lecture
        serial.write(8); Sleep(5); serial.write(8+2+1); Sleep(5);      // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp
        Sleep(50);

        // lecture du buffer de 4 octets
        for(i=0;i<4;i++)
        {
            //serial.write(2); Sleep(5); serial.write(0xd0+i); Sleep(5); // set ADR
            //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            data=uu_read(0xd0+i);
            printf("\ncc_%d: %d",4*j+i,data);
            fprintf(pf,"\ncc_%d: ",4*j+i);
            if((4*j+i)<10) fprintf(pf," ");
            if((4*j+i)<100) fprintf(pf," ");
            fprintf(pf,"%d",data);
        }
    }


    // 3. Lecture de la table des itineraires
    printf("\n- 3 Lecture de la table des itineraires");
    fprintf(pf,"\n\n[iti]");
    fprintf(pf,"\n# aig   00000000 00111111 11112222 22222233  .=non utilise -=direct");
    fprintf(pf,"\n#       01234567 89012345 67890123 45678901                /=devie ");
    serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2/3 (mode.0=1)
    for(j=0;j<(32*2);j++)
    {
        // preparation de la lecture
        serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);       // adr = fl_adrh
        serial.write(3); Sleep(5); serial.write(0x15); Sleep(5);       // fl_adrh=0

        serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);       // adr = fl_adrl 00
        serial.write(3); Sleep(5); serial.write((4*j)&255); Sleep(5);  // fl_adrl=4*j

        serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);       // adr = fl_nb
        serial.write(3); Sleep(5); serial.write(4); Sleep(5);          // fl_nb = 4

        // ordre de lecture
        serial.write(8); Sleep(5); serial.write(2+1); Sleep(5);      // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp
        Sleep(50);

        // lecture du buffer de 4 octets
        for(i=0;i<4;i++)
        {
            if(i==0 && (j%2)==0 ) printf("\niti_%d: ",j/2);
            //if(i==0 && (j%2)==0 ) { fprintf(pf,"\niti_%d: ",j/2);  if((j/2)<10) fprintf(pf," "); }
            //serial.write(2); Sleep(5); serial.write(0xd0+i); Sleep(5); // set ADR
            //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            data=uu_read(0xd0+i);
            printf("%02x.",data);
            //fprintf(pf,"%02x.",data);
            buff[4*(j%2)+i]=data;
        }
        if((j%2)==1)
        {
            fprintf(pf,"\niti_%d:",j/2); if((j/2)<10) fprintf(pf," ");
            for(k=0;k<4;k++) for(l=0;l<8;l++)
            {
                if(l==0) fprintf(pf," ");
                if((buff[2*k]&(1<<l))==0) fprintf(pf,".");
                else
                {
                    if((buff[2*k+1]&(1<<l))==0) fprintf(pf,"-");
                    else                        fprintf(pf,"/");
                }
            }
        }

    }

    // 4. Lecture de la table des equations
    printf("\n- 4 Lecture de la table des equations");
    fprintf(pf,"\n\n[eq]");
    serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2/3 (mode.0=1)
    for(j=0;j<128;j++)
    {
        // preparation de la lecture
        serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);                // adr = fl_adrh
        serial.write(3); Sleep(5); serial.write(0x16+((4*j)/256)); Sleep(5);    // fl_adrh=0

        serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);                // adr = fl_adrl 00
        serial.write(3); Sleep(5); serial.write((4*j)&255); Sleep(5);           // fl_adrl=4*j

        serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);                // adr = fl_nb
        serial.write(3); Sleep(5); serial.write(4); Sleep(5);                   // fl_nb = 4

        // ordre de lecture
        serial.write(8); Sleep(5); serial.write(2+1); Sleep(5);                 // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp
        Sleep(50);

        // lecture du buffer de 4 octets
        for(i=0;i<4;i++)
        {
            if(i==0) printf("\neq_%d: ",2*j);
          //if(i==0) { fprintf(pf,"\neq_%d: ",2*j); if((2*j)<10) fprintf(pf," "); if((2*j)<100) fprintf(pf," "); }
            if(i==2) printf("\neq_%d: ",2*j+1);
          //if(i==2) { fprintf(pf,"\neq_%d: ",2*j+1); if((2*j+1)<10) fprintf(pf," "); if((2*j+1)<100) fprintf(pf," "); }
          //serial.write(2); Sleep(5); serial.write(0xd0+i); Sleep(5); // set ADR
          //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            data=uu_read(0xd0+i);
            printf("%02x.",data);
          //fprintf(pf,"%02x.",data);
            buff[i]=data;
        }
        for(i=0;i<2;i++)
        {
            fprintf(pf,"\neq_%d: ",2*j+i); if((2*j)<10) fprintf(pf," "); if((2*j)<100) fprintf(pf," ");
            eq_ordre_val_copute_str(buff[2*i],buff[2*i+1],sz_str);
            fprintf(pf,"%s",sz_str);
        }
    }

    // 7. Cloture
    printf("\n- 7 Fermeture du fichier et de la liaison serie");
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(5);
    if(data!=0x83) {printf("\nErreur: Centrale non detectee\n"); return; }
    serial.close();
    fclose(pf);
    printf("\nLa configuration de la centrale se trouve dans le fichier fdcc_conf_lect.txt");
}



int write_buff_eeflash(int write_buff[], int adr, int ee)
{
    int buff[4];
    int i,data,ok;


    if((write_buff[0]+write_buff[1]+write_buff[2]+write_buff[3])==-4) return 1;

    serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2/3 (mode.0=1)

    // LECTURE pour voir si besoin d'ecrire

    printf(" lecture");

    // preparation de la lecture
    serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);       // adr = fl_adrh
    serial.write(3); Sleep(5); serial.write(adr/256); Sleep(5);          // fl_adrh=0

    serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);       // adr = fl_adrl 00
    serial.write(3); Sleep(5); serial.write(adr&255); Sleep(5);    // fl_adrl=4*j

    serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);       // adr = fl_nb
    serial.write(3); Sleep(5); serial.write(4); Sleep(5);          // fl_nb = 4

    // ordre de lecture
    serial.write(8); Sleep(5); serial.write(8*ee+2+1); Sleep(50);  // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp

    // lecture du buffer de 4 octets
    for(i=0;i<4;i++)
    {
        //serial.write(2); Sleep(5); serial.write(0xd0+i); Sleep(5); // set ADR
        //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
        data=uu_read(0xd0+i);
        buff[i]=data;
    }

    // si identique alors pas d'ecriture
    if(  (buff[0]==write_buff[0])
       &&(buff[1]==write_buff[1])
       &&(buff[2]==write_buff[2])
       &&(buff[3]==write_buff[3]) ) return 1;

    // ECRITURE

    printf(" ecriture");

    // preparation a l'ecriture
    serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);       // adr = fl_adrh
    serial.write(3); Sleep(5); serial.write(adr/256); Sleep(5);    // fl_adrh=0

    serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);       // adr = fl_adrl 00
    serial.write(3); Sleep(5); serial.write(adr&255); Sleep(5);    // fl_adrl=4*j

    serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);       // adr = fl_nb
    serial.write(3); Sleep(5); serial.write(4); Sleep(5);          // fl_nb = 4

    serial.write(2); Sleep(5); serial.write(0xa7); Sleep(5);       // adr = rs_w_protect1
    serial.write(3); Sleep(5); serial.write(0xaa); Sleep(5);       // = 0xaa

    serial.write(2); Sleep(5); serial.write(0xa8); Sleep(5);       // adr = rs_w_protect2
    serial.write(3); Sleep(5); serial.write(0x55); Sleep(5);       // = 0x55

    // ecriture dans buffer de 4 octets
    for(i=0;i<4;i++)
    {
        data=write_buff[i];
        if(data!=-1)
        {
            serial.write(2); Sleep(5); serial.write(0xd0+i); Sleep(5); // set ADR
            serial.write(3); Sleep(10); serial.write(data);  Sleep(10); // read
        }
    }

    // ordre d'ecriture
    //printf("\nEcriture cc_%d-%d",4*j,4*j+3);
    serial.write(8); Sleep(5); serial.write(8*ee+4+1); Sleep(100); // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp

    serial.write(2); Sleep(5); serial.write(0xa7); Sleep(5);       // adr = rs_w_protect1
    serial.write(3); Sleep(5); serial.write(0); Sleep(5);          // = 0

    serial.write(2); Sleep(5); serial.write(0xa8); Sleep(5);       // adr = rs_w_protect2
    serial.write(3); Sleep(5); serial.write(0); Sleep(5);          // = 0

    // LECTURE pour verification de l'ecriture

    printf(" verification");

    // preparation de la lecture
    serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);       // adr = fl_adrh
    serial.write(3); Sleep(5); serial.write(adr/256); Sleep(5);    // fl_adrh=0

    serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);       // adr = fl_adrl 00
    serial.write(3); Sleep(5); serial.write(adr&255); Sleep(5);    // fl_adrl=4*j

    serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);       // adr = fl_nb
    serial.write(3); Sleep(5); serial.write(4); Sleep(5);          // fl_nb = 4

    // ordre de lecture
    serial.write(8); Sleep(5); serial.write(8*ee+2+1); Sleep(50);  // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp

    // lecture du buffer de 4 octets
    for(i=0;i<4;i++)
    {
        //serial.write(2); Sleep(5); serial.write(0xd0+i); Sleep(5); // set ADR
        //serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
        data=uu_read(0xd0+i);
        buff[i]=data;
    }

    // verification de l'ecriture
    ok=1;
    for(i=0;i<4;i++) if(write_buff[i]!=-1) if(buff[i]!=write_buff[i]) ok=0;
    if(ok==0) { printf(" ERREUR"); return 0; }
    printf(" OK"); return 1;
}



void write_conf(void)
{
    FILE* pf;
    int i,j,nb,data,sect,ok,p;
    int buff[4];
    char c;
    char sz_str[1024];

    int tab_write_cc[256];
    int tab_write_iti[256];
    int tab_write_eq[512];
    int tab_write_depl[1000];
    int tab_write_seq[1024];

    int i_eq,i_seq,ordre,ret;
    char sz_ordre[1024];
    int hour,hour_b,min,min_b,b,var;

    // 1. Initialisations
    printf("\n- 1 Ouverture du fichier et de la liaison serie");

    SelectAutoFileOpen();
    pf=fopen(szOpenFile,"rt");
    //pf=fopen("fdcc_conf_ecr.txt","rt");
    if(pf==NULL) {printf("\nErreur: Ne peut pas ouvrir le fichier %s\n",szOpenFile); return; }
    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(100); serial.read(&data);  Sleep(10);
    if(data!=0x83) {printf("\nErreur: Centrale non detectee (%x)\n",data); return; }

    // 2. Remplir tab_write... a partir du fichier
    for(i=0;i<256;i++)  tab_write_cc[i]=-1;
    for(i=0;i<256;i++)  tab_write_iti[i]=-1;
    for(i=0;i<512;i++)  tab_write_eq[i]=-1;
    for(i=0;i<1000;i++) tab_write_depl[i]=-1;
    for(i=0;i<1024;i++) tab_write_seq[i]=-1;
    i_eq=0; ordre=1; i_seq=-1;
    printf("\n- 2 Lecture du fichier");
    while(fscanf(pf," %s",sz_str)!=EOF)
    {
        if(sz_str[0]=='#') { while(fscanf(pf,"%c",&c)!=EOF) {if(c=='\r') break; if(c=='\n') break; } continue; }
        if(sz_str[0]=='[')
        {
            sect=0;
            if(!stricmp(sz_str,"[cc]")) sect=1;
            if(!stricmp(sz_str,"[iti]")) sect=2;
            if(!stricmp(sz_str,"[eq]")) sect=3;
            if(!stricmp(sz_str,"[depl]")) sect=4;
            if(!stricmp(sz_str,"[seq]")) sect=5;
            continue;
        }

        if(sect==1)
        {
             if(strnicmp(sz_str,"cc_",3)) { printf("\nErreur: %s in section [eq]",sz_str); return; }
             nb=atoi(&sz_str[3]); if( nb<0 || nb>255 ) { printf("\nErreur: %s in section [eq]",sz_str); return; }
             fscanf(pf," %d",&data); if( data<0 || data>255 ) { printf("\nErreur: %s in section [eq]",sz_str); return; }
             tab_write_cc[nb]=data;
             printf("\ncc_%d: %d",nb,data);
        }

        if(sect==2)
        {
             if(strnicmp(sz_str,"iti_",4)) { printf("\nErreur0: %s in section [iti]",sz_str); return; }
             nb=atoi(&sz_str[4]); if( nb<0 || nb>31 ) { printf("\nErreur1: %s in section [iti]",sz_str); return; }
             printf("\niti_%d: ",nb);
             for(i=0;i<8;i++) tab_write_iti[8*nb+i]=0;
             i=0;
             do
             {
                 fscanf(pf," %c",&c);
                 switch(c)
                 {
                     case '.': i++;
                               break;
                     case '-': tab_write_iti[8*nb+2*(i/8)]|=1<<(i%8); //mask
                               i++;
                               break;
                     case '/': tab_write_iti[8*nb+2*(i/8)]|=1<<(i%8); //mask
                               tab_write_iti[8*nb+2*(i/8)+1]|=1<<(i%8); //pos_deviee
                               i++;
                               break;
                     default:  printf("\nErreur2: %s in section [iti] >%c<",sz_str,c); return; break;
                 }
             }
             while(i!=32);

             for(i=0;i<8;i++) printf("%02x.",tab_write_iti[8*nb+i]);
        }

        if(sect==3) //eq
        {
             if(0==strnicmp(sz_str,"eq",2)) continue;

             if(i_eq==256) { printf("\nErreur: trop d'equations (256 eq max)"); return; }

             if(ordre==1)
             {
                  if(     0==stricmp(sz_str,"end")) { tab_write_eq[2*i_eq]=15<<2; tab_write_eq[2*i_eq+1]=0xff; i_eq++; }
                  else if(0==stricmp(sz_str,"nop")) { tab_write_eq[2*i_eq]=14<<2; tab_write_eq[2*i_eq+1]=0xff; i_eq++; }
                  else if(0==stricmp(sz_str,"inv")) { tab_write_eq[2*i_eq]=12<<2; tab_write_eq[2*i_eq+1]=0xff; i_eq++; }
                  else { ordre=0; strcpy(sz_ordre,sz_str); }
             }
             else
             {
                 ret=eq_szordre_szvar_to_nordre_nvar(sz_ordre,sz_str,&tab_write_eq[2*i_eq],&tab_write_eq[2*i_eq+1]);
                 if(ret==0) { printf("\nErreur: in eq: %s %s",sz_ordre,sz_str); return; }
                 i_eq++;
                 ordre=1;
             }

             if(ordre==1) printf("\neq: [%02x,%02x]",tab_write_eq[2*i_eq-2],tab_write_eq[2*i_eq-1]);
        }
    }

    // 3. verifs
    printf("\n- 3 verification du tableau");
    for(j=0;j<256;j++)   printf("\ncc[%d]=%d",j,tab_write_cc[j]);
    for(j=0;j<32 ;j++) { printf("\niti[%d]=" ,j); for(i=0;i<8 ;i++) printf("%02x.",tab_write_iti[8*j+i]);  }
    for(j=0;j<256;j++) { printf("\neq[%d]="  ,j); for(i=0;i<2 ;i++) printf("%02x.",tab_write_eq[2*j+i]);   }


    // 4. cc
    printf("\n- 4 ecriture des CC en EEPROM");
    for(j=0;j<64;j++)
    {
        printf("\ncc_%d-%d",4*j,4*j+3);
        write_buff_eeflash(&tab_write_cc[4*j],4*j,1);
    }

    // 5. iti
    printf("\n- 5 ecriture des itineraires en flash");
    for(j=0;j<32;j++) for(i=0;i<2;i++)
    {
        printf("\niti_%d (block%d) ",j,i+1);
        write_buff_eeflash(&tab_write_iti[8*j+4*i],0x1500+8*j+4*i,0);
    }

    // 6. eq
    printf("\n- 6 ecriture des equations en flash");
    for(j=0;j<128;j++)
    {
        printf("\neq_%d-%d",2*j,2*j+1);
        write_buff_eeflash(&tab_write_eq[4*j],0x1600+4*j,0);
    }

    // 9. Cloture
    printf("\n- 9 Fermeture du fichier et de la liaison serie");
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(5);
    if(data!=0x83) {printf("\nErreur: Centrale non detectee\n"); return; }
    serial.close();
    fclose(pf);
    printf("\nFin de la programmation");
}



void read_buff_ee(int adr,int* buff)
{
        serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2/3 (mode.0=1)

        // preparation de la lecture
        serial.write(2); Sleep(5); serial.write(0xe8); Sleep(5);       // adr = fl_adrh
        serial.write(3); Sleep(5); serial.write(0); Sleep(5);          // fl_adrh=0

        serial.write(2); Sleep(5); serial.write(0xe9); Sleep(5);       // adr = fl_adrl 00
        serial.write(3); Sleep(5); serial.write(adr); Sleep(5);        // fl_adrl=adr

        serial.write(2); Sleep(5); serial.write(0xeb); Sleep(5);       // adr = fl_nb
        serial.write(3); Sleep(5); serial.write(4); Sleep(5);          // fl_nb = 1

        // ordre de lecture
        serial.write(8); Sleep(5); serial.write(8+2+1); Sleep(5);      // mode.3=8=eeprom mode.2=4=wr mode.1=2=rd mode.0=1=irp
        Sleep(50);

        // lecture du buffer
        //serial.write(2); Sleep(5); serial.write(0xd0); Sleep(5); // set ADR
        //serial.write(4); Sleep(10); serial.read(buff);  Sleep(10); // read
        *buff=uu_read(0xd0);

        serial.write(8); Sleep(10); serial.write(0); Sleep(10); // clr IRF
}



void read_ee(void)
{
    int i,data,adr;

    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    while(1)
    {
        printf("\nnumero de l'element de config EE (-1=fin) -> ");
        scanf(" %d",&adr);
        if(adr<0 || adr>255) break;

        read_buff_ee(adr,&data);
        printf("EE[%d]=%d",adr,data);
    }

    serial.close();
}



void write_ee(void)
{
    int buff[4];
    int i,data,adr;

    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    while(1)
    {
        printf("\nnumero de l'element de config EE (-1=fin) -> ");
        scanf(" %d",&adr);
        if(adr<0 || adr>255) break;
        printf("valeur -> ");
        scanf(" %d",&data);
        if(data<0 || data>255) break;

        //implementation temporaire (bloc de 4)
        for(i=0;i<4;i++)
        {
            read_buff_ee(4*(adr/4)+i,&buff[i]);
            printf("\nEE[%d]=%d",4*(adr/4)+i,buff[i]);
        }

       printf("\n->");
       buff[adr%4]=data;
       write_buff_eeflash(buff, 4*(adr/4), 1);

        for(i=0;i<4;i++)
        {
            read_buff_ee(4*(adr/4)+i,&buff[i]);
            printf("\nEE[%d]=%d",4*(adr/4)+i,buff[i]);
        }
    }

    serial.close();

   // write_buff_eeflash(int write_buff[], int adr, int ee);
}



void chg_ch_en(void)
{
    int i,data,data1,data2,adr,ch;
    int v_souris,v_cli,v_um,v_tdv;

    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    uu_irf(0);

    // lecture du buffer
    //  en_ch_0_7      EQU     0x71  ;0123 *  (INIT_EE)
    //  en_ch_8_15     EQU     0x72  ;0123 *  (INIT_EE)

    data1=uu_read(0x71);
    data2=uu_read(0x72);

    data=uu_read(0x70);
    v_um=(data&8)/8;
    v_cli=(data&4)/4;
    v_tdv=(data&2)/2;
    v_souris=(data&1)/1;

    while(1)
    {
        system("cls");
        printf("\nModification temporaire des activations\n(utilisez les variables EE pour les rendre definitive)");
        printf("\nconfig actuelle (1=souris/um,-=rs232)");
        for(i=0;i<8;i++)
        {
            if(data1&(1<<i)) printf("\n ch%2d = 1",i);   else printf("\n ch%2d = -",i);
            if(data2&(1<<i)) printf("   ch%2d = 1",i+8); else printf("   ch%2d = -",i+8);
        }
        printf("\n 20:souris = %d",v_souris);
        printf("\n 21:um     = %d",v_um);
        printf("\n 22:tdv    = %d",v_tdv);
        printf("\n 23:cli    = %d",v_cli);

        printf("\ncanal a modifier (-1=fin) --> "); scanf(" %d",&ch);

        if((ch<=-1) || (ch>=16 && ch<=19) || (ch>=24)) break;

        if(ch>=0 && ch<=15)
        {
            if(ch<=7) { data1^=1<<ch;     uu_write(0x71,data1); }
            else      { data2^=1<<(ch-8); uu_write(0x72,data2); }
        }
        if(ch>=20 && ch<=23)
        {
            if(ch==20) v_souris^=1;
            if(ch==21) v_um^=1;
            if(ch==22) v_tdv^=1;
            if(ch==23) v_cli^=1;
            data=0;
            if(v_souris) data|=1;
            if(v_tdv)    data|=2;
            if(v_cli)    data|=4;
            if(v_um)     data|=8;
            uu_write(0x70,data);
        }
    }

    serial.close();
}





/*****************************************************************************/
/* CV                                                                        */
/*****************************************************************************/


void main_prog_cv(void)
{
    int i,data,code,cv_adr,cv_dat,cv_page,cv_reg;

    serial.init(sz_com,19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    printf("\n============================================================================");
    printf("\n= ATTENTION, Veuillez vous assurer qu'il n'y ai qu'une locomotive presente =");
    printf("\n=                          (car toutes les locomotives seront programmees) =");
    printf("\n============================================================================");
    printf("\nVeuillez saisir 1234 pour continuer\n--> ");
    scanf(" %d",&code); if(code!=1234) return;

/*
rs_config     EQU     0x77   ;0123   ici temporairement pour acces ds toutes les banks
dcc_p_pag     EQU     0x3c   ;0---
dcc_p_reg     EQU     0x3d   ;0---
dcc_p_dat     EQU     0x3e   ;0---
state_au      EQU     0x2d   ;--2- 0.PROG_EN.PROG_WO_IT.CHG.CLR.SET.BPOLD.AU
*/

    while(1)
    {
        printf("\n\nEntrez l'adresse et la donnee du CV en decimal (0 pour terminer)");
        printf("\nCV --> "); scanf(" %d", &cv_adr); if(cv_adr<=0) break;
        printf("DAT --> "); scanf(" %d", &cv_dat);

        // calcul de la page et du registre (page=1,reg=1)=cv1  cv=4*(page-1)+(reg+1)
        cv_page=(((cv_adr-1)/4)+1);
        cv_reg=((cv_adr-1)%4)+1;

        printf("\nprog cv%d (page=%d,reg=%d) = %d",cv_adr,cv_page,cv_reg,cv_dat);

        uu_irf(0);              // clr IRF pour bank0
        uu_write(0x3c,cv_page);
        uu_write(0x3d,cv_reg);
        uu_write(0x3e,cv_dat);
        uu_irf(1);              // set IRF pour bank2
        uu_write(0x2d,0x60);    // set ADR (state_au) // write deprotection (bit 6), start prog (bit 5)
        Sleep(10000);           // maisse le temps a la centrale de programmer le decodeur
        uu_write(0x2d,0);       // write protection (bit 6), start prog (bit 5) se remet a 0 automatiquement
        uu_irf(0);              // clr IRF pour bank0
    }

    serial.close();
}




/*****************************************************************************/
/* MENU                                                                      */
/*****************************************************************************/


int main(int argc, char *argv[])
{
    int choix;

    GetCurrentDirectory(999,szLaunchDir);

    do
    {
      //system("cls");
        printf("\n\n==============================================");
        printf("\n= rs232_fdcc_2008_conf (version 17 mars 2008)=");
        printf("\n==============================================");
        printf("\nLIAISON");
        printf("\n1 = changement du port RS232 (actuellement COM%d)",icom);
        printf("\n2 = test de la liaison");
        printf("\n\nCONFIG DE LA CENTRALE");
        printf("\n3 = lecture  de la config ee/iti/tdv (centrale -> fichier )"); // (centrale -> fichier fdcc_conf_lect.txt)
        printf("\n4 = ecriture de la config ee/iti/tdv (fichier  -> centrale)"); // (fichier fdcc_conf_ecr.txt - > centrale)");
        printf("\n5 = lecture  d'une variable de configuration ee");
        printf("\n6 = ecriture d'une variable de configuration ee");
        printf("\n7 = activation / desactivation (canaux souris / rs232)(tdv/um/cli)");
        printf("\n\nDECODEURS");
        printf("\n8 = programmation d'un CV");
      //printf("\nPILOTAGE AUTOMATIQUE");
      //printf("\n9 = pilotage automatique sequenciel (fichier fdcc_conf_seq.txt)");
        printf("\n\nDEBUG");
        printf("\n10 = affichage des entrees");
        printf("\n11 = affichage des canaux");
        printf("\n12 = observation de la memoire RAM");
        printf("\n13 = test aig out led");
        printf("\n14 = test des souris");
        printf("\n\nchoix (0 pour fin) -> ");
        scanf(" %d",&choix);

        system("cls");
        switch(choix)
        {
            case 1 : select_port();        break;
            case 2 : test_link();          break;
            case 3 : read_conf();          break;
            case 4 : write_conf();         break;
            case 5 : read_ee();            break;
            case 6 : write_ee();           break;
            case 7 : chg_ch_en();          break;
            case 8 : main_prog_cv();       break;
          //case 9 : main_auto_file();     break;
            case 10: main_disp_inputs();   break;
            case 11: main_disp_canaux();   break;
            case 12: main_dump_ram();      break;
            case 13: modify_aig_out_led(); break;
            case 14: test_mouse();         break;
            default: break;
        }
      //Sleep(1000);
    }
    while (choix!=0);

    return 0;
}















































































































































/*****************************************************************************/
/* DEPLACEMENTS AUTOMATIQUES  (no more used with DOS because on Windows now  */
/*****************************************************************************/


int rs232_adr[]={0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0  };
int rs232_cmd[]={0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60 }; // 01DVVVVV D=1=av D=0=arr V0.V4.V3.V2.V1
int rs232_fct[]={0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90 }; // 100.FL.F4.F3.F2.F1
int rs232_led[]={0,0,0,0,0,0,0,0};
int rs232_cli[]={0,0,0,0,0,0,0,0};
int rs232_aig[]={0,0,0,0};
int rs232_out[]={0,0,0,0};

int rs232_in[8];
int rs232_pos[8];
int rs232_mouse[16];
int rs232_can[5];


void txrx_masse(void)
{
    int i,data,pr;
    static int cpt=0;

    serial.write(9); Sleep(10); //transfert de masse

    // PC->UC: ADR(16)+CMD(16)+FCT(16)+LED(8)+CLI(8)+AIG(4)+OUT(4)
    // UC->PC: IN_MEM(8)+AIG_POS(8)+MOUSE_W(16)+can%8_pour_le_reste

    for(i=0;i<72;i++)
    {
        if(i>=0 && i<=15) data=rs232_adr[i-0];
        if(i>=16 && i<=31) data=rs232_cmd[i-16];
        if(i>=32 && i<=47) data=rs232_fct[i-32];
        if(i>=48 && i<=55) data=rs232_led[i-48];
        if(i>=56 && i<=63) data=rs232_cli[i-56];
        if(i>=64 && i<=67) data=rs232_aig[i-64];
        if(i>=68 && i<=71) data=rs232_out[i-68];

        serial.write(data); Sleep(2);
    }
    Sleep(10);
    //printf("\r");
    ////printf("\n");
    for(i=0;i<72;i++)
    {
        serial.read(&data);
        pr=0;

        if(i>=0  && i<=7 ) { rs232_in[i-0]=data; pr=1; }
        if(i>=8  && i<=15) { rs232_pos[i-8]=data; pr=1; }
        if(i>=16 && i<=31) { rs232_mouse[i-16]=data; pr=0; } //plus utilise
        if(i>=32 && i<=36) { rs232_can[i-32]; pr=1; }        //plus utilise

        ////if(pr) printf("%02x.",data);
    }
    ////printf("%04d",cpt++);

    Sleep(86); // pour que la fct dure 250ms environ
}



void main_auto_file(void)
{
/*
Syntax:
#commentaire
start

depl <numero(0-999)>
loco <adr(0-99)>                        #si loco libre (sinon bloque)
//aig /-.!.... ........ ........ ........ # -=droit /=devie .=non utilise !=reserver mais ne pas bouger (pour l'instant)
aig <num> <-//>
vit <+/-> <vit(0-31)>
fct <num(0-4)> <val(0/1)>
tempos <time_1s(0-1000)>
tempot <time_tic_0.25s(0-1000)>
wait_in <num>
sound "<chemin d'acces>"

seq
run_wait_depl 0
sound "<chemin d'acces>"
tempos <time_1s(0-1000)>

stop
end
*/

    int i,j,data,data2,cpt,step,tempo;
    FILE* pf;
    int wait=0;
    char sz_ch[200];
    char sz_ordre[200];
    char sz_sound[1000];
    int loco=0;
    int vit,val,cmd,fct,in,msk,aig,delay,start_ok=0;
    char c;
    int depl_no=0, depl_torun;
    int section_on;
    int ordre;
    int e;

    #define DEPL_LIST_NB 100000
    int depl_list[DEPL_LIST_NB];  /* ORDRE puis parametre(s) */
    int depl_list_next;
    #define DEPL_ORDRE_FREE      0
    #define DEPL_ORDRE_DEPL      1
    #define DEPL_ORDRE_LOCO      2
    #define DEPL_ORDRE_AIG       3
    #define DEPL_ORDRE_VIT       4
    #define DEPL_ORDRE_FCT       5
    #define DEPL_ORDRE_TEMPOS    6
    #define DEPL_ORDRE_TEMPOT    7
    #define DEPL_ORDRE_WAITIN    8
    #define DEPL_ORDRE_WAITNOTIN 9
    #define DEPL_ORDRE_SOUND     10
    #define DEPL_ORDRE_ECHO      11

    #define DEPL_NB 1000
    int depl[DEPL_NB];
    #define DEPL_FREE -1

    #define SEQ_LIST_NB 10000
    int seq_list[SEQ_LIST_NB];
    int seq_list_next;
    int pseq;
    #define SEQ_ORDRE_FREE     0
    #define SEQ_ORDRE_RUNWAIT  1
    #define SEQ_ORDRE_RUN      2
    #define SEQ_ORDRE_ENDRUN   3
    #define SEQ_ORDRE_TEMPOS   4
    #define SEQ_ORDRE_SOUND    5
    #define SEQ_ORDRE_ECHO     6

    #define EXE_NB 6
    int exe[EXE_NB][10];
    #define EXE_DEPL  0    // (-1=inactif)
    #define EXE_PTR   1
    #define EXE_LOCO  2
    #define EXE_WAIT  3

    // 1. initialisation des structures
    for(i=0;i<DEPL_LIST_NB;i++) depl_list[i]=DEPL_ORDRE_FREE;  depl_list_next=0;
    for(i=0;i<DEPL_NB;i++) depl[i]=DEPL_FREE;
    for(i=0;i<SEQ_LIST_NB;i++) seq_list[i]=SEQ_ORDRE_FREE; seq_list_next=0;
    for(i=0;i<EXE_NB;i++) for(j=0;j<10;j++) exe[i][j]=-1;

    // 2. remplissage en provenance du fichier
    pf=fopen("fdcc_conf_seq.txt","rt");
    if(pf==NULL) { printf("\nCannot open fdcc_conf_seq.txt\nBye\n\n"); exit(1); }


    printf("\n- filling stuctures");
    section_on=1;
    depl_no=-2; //mode seq
    while(1)
    {
        if(fscanf(pf," %s",sz_ordre)==EOF) break;
        //printf("\nordre=%s",sz_ordre);

        if((sz_ordre[0]=='#') || (sz_ordre[0]=='/'))
        {
        while(1)
        {
           if(fscanf(pf,"%c",&c)==EOF) break;
           if(c=='\n') break;
           if(c=='\r') break;
        }
        continue;
        }

        if(stricmp(sz_ordre,"section_on")==0) section_on=1;
        if(stricmp(sz_ordre,"section_off")==0) section_on=0;

        if(section_on==0) continue;

        printf("\nordre=%s",sz_ordre);

        //if(stricmp(sz_ordre,"end")==0) break;
        //if(stricmp(sz_ordre,"stop")==0) break;

        if(stricmp(sz_ordre,"depl")==0)  // loco 0
        {
            if(fscanf(pf," %d",&depl_no)==EOF) break;
            //printf(" depl_no=%d\n",depl_no);
            depl[depl_no]=depl_list_next;
            depl_list[depl_list_next]=DEPL_ORDRE_DEPL; depl_list_next++;
            depl_list[depl_list_next]=depl_no;         depl_list_next++;

        }

        if(stricmp(sz_ordre,"seq")==0) depl_no=-2;  // loco 0

        if(stricmp(sz_ordre,"run_wait_depl")==0)  //
        {
            if(fscanf(pf," %d",&depl_torun)==EOF) break;
            seq_list[seq_list_next]=SEQ_ORDRE_RUNWAIT; seq_list_next++;
            seq_list[seq_list_next]=depl_torun; seq_list_next++;
        }

        if(stricmp(sz_ordre,"run_depl")==0)  //
        {
            if(fscanf(pf," %d",&depl_torun)==EOF) break;
            seq_list[seq_list_next]=SEQ_ORDRE_RUN; seq_list_next++;
            seq_list[seq_list_next]=depl_torun; seq_list_next++;
        }

        if(stricmp(sz_ordre,"wait_end_depl")==0)  //
        {
            if(fscanf(pf," %d",&depl_torun)==EOF) break;
            seq_list[seq_list_next]=SEQ_ORDRE_ENDRUN; seq_list_next++;
            seq_list[seq_list_next]=depl_torun; seq_list_next++;
        }

        if(stricmp(sz_ordre,"loco")==0)  // loco 0
        {
            if(fscanf(pf," %d",&loco)==EOF) break;
            //printf(" loco=%d\n",loco);
            depl_list[depl_list_next]=DEPL_ORDRE_LOCO; depl_list_next++;
            depl_list[depl_list_next]=loco;            depl_list_next++;
        }

        if(stricmp(sz_ordre,"vit")==0)  // vit + 20
        {
            if(fscanf(pf," %c",&c)==EOF) break;
            if(fscanf(pf," %d",&vit)==EOF) break;
            //printf(" vit= %c %d\n",c,vit);
            depl_list[depl_list_next]=DEPL_ORDRE_VIT; depl_list_next++;
            depl_list[depl_list_next]=c;              depl_list_next++;
            depl_list[depl_list_next]=vit;            depl_list_next++;
            //cmd=(vit>>1)&0x0f;
            //if(vit&1) cmd|=0x10;
            //if(c=='+') cmd|=0x20;
            //cmd|=0x40;
            //rs232_cmd[loco]=cmd;
        }

        if(stricmp(sz_ordre,"fct")==0)  // fct 0 1
        {
            if(fscanf(pf," %d",&fct)==EOF) break;
            if(fscanf(pf," %d",&val)==EOF) break;
            //printf(" fct(%d)=%d\n",fct,val);
            depl_list[depl_list_next]=DEPL_ORDRE_FCT; depl_list_next++;
            depl_list[depl_list_next]=fct;            depl_list_next++;
            depl_list[depl_list_next]=val;            depl_list_next++;
            //msk=(1<<fct); if(msk&1) msk|=0x20; msk>>1;   //F0-F4.3.2.1
            //rs232_fct[loco]&=(0xff^msk);
            //if(val) rs232_fct[loco]|=msk;
        }

        if(stricmp(sz_ordre,"aig")==0)  // aig 3 -
        {
            if(fscanf(pf," %d",&aig)==EOF) break;
            if(fscanf(pf," %c",&c)==EOF) break;
            //printf(" aig(%d)=%c\n",aig,c);
            depl_list[depl_list_next]=DEPL_ORDRE_AIG; depl_list_next++;
            depl_list[depl_list_next]=aig;            depl_list_next++;
            depl_list[depl_list_next]=c;              depl_list_next++;
            //if(c=='-') rs232_aig[aig/8]&=(0xff^(1<<(aig%8)));    //droit
            //if(c=='/') rs232_aig[aig/8]|=1<<(aig%8);             //devie
        }

        if(stricmp(sz_ordre,"echo")==0)  // echo "blabla...."
        {
            if(depl_no>=0 ) { depl_list[depl_list_next]=DEPL_ORDRE_ECHO; depl_list_next++; }
            if(depl_no==-2) { seq_list[seq_list_next  ]=SEQ_ORDRE_ECHO;  seq_list_next++;  }
            fscanf(pf," %c",&c); // remove first \"
            while(1)
            {
                if(fscanf(pf,"%c",&c)==EOF) break;
                printf("%c",c);
                if(c=='\"') c='\0';
                if(depl_no>=0 ) { depl_list[depl_list_next]=c; depl_list_next++; }
                if(depl_no==-2) { seq_list[seq_list_next  ]=c;  seq_list_next++; }
                if(c=='\0') break;
            }
        }

        if(stricmp(sz_ordre,"sound")==0)  // sound "c:\...."
        {
            if(depl_no>=0 ) { depl_list[depl_list_next]=DEPL_ORDRE_SOUND; depl_list_next++; }
            if(depl_no==-2) { seq_list[seq_list_next  ]=SEQ_ORDRE_SOUND;  seq_list_next++;  }
            fscanf(pf," %c",&c); // remove first \"
            while(1)
            {
                if(fscanf(pf,"%c",&c)==EOF) break;
                printf("%c",c);
                if(c=='\"') c='\0';
                if(depl_no>=0 ) { depl_list[depl_list_next]=c; depl_list_next++; }
                if(depl_no==-2) { seq_list[seq_list_next  ]=c;  seq_list_next++; }
                if(c=='\0') break;
            }
            //ShellExecute(NULL,"open",sz_sound, NULL, NULL, SW_SHOWNA);
        }

        if(stricmp(sz_ordre,"tempot")==0)  // tempot 40  (10s)
        {
            if(fscanf(pf," %d",&delay)==EOF) break;
            //printf(" tempot=%d\n",delay);
            depl_list[depl_list_next]=DEPL_ORDRE_TEMPOT;  depl_list_next++;
            depl_list[depl_list_next]=delay;              depl_list_next++;
        }

        if(stricmp(sz_ordre,"tempos")==0)  // tempos 10  (10s)
        {
            if(fscanf(pf," %d",&delay)==EOF) break;
            //printf(" tempos=%d\n",delay);
            if(depl_no>=0)
            {
                depl_list[depl_list_next]=DEPL_ORDRE_TEMPOS; depl_list_next++;
                depl_list[depl_list_next]=delay;             depl_list_next++;
            }
            if(depl_no==-2) // sequenceur
            {
                seq_list[seq_list_next]=SEQ_ORDRE_TEMPOS; seq_list_next++;
                seq_list[seq_list_next]=delay;            seq_list_next++;
            }
        }

        if(stricmp(sz_ordre,"wait_in")==0)  // wait_in 8
        {
            if(fscanf(pf," %d",&in)==EOF) break;
            //printf(" wait_in=%d\n",in);
            depl_list[depl_list_next]=DEPL_ORDRE_WAITIN;  depl_list_next++;
            depl_list[depl_list_next]=in;                 depl_list_next++;
            //if(rs232_in[in/8]&(1<<(in%8))) wait=0;
        }

        if(stricmp(sz_ordre,"wait_not_in")==0)  // wait_in 8
        {
            if(fscanf(pf," %d",&in)==EOF) break;
            //printf(" wait_not_in=%d\n",in);
            depl_list[depl_list_next]=DEPL_ORDRE_WAITNOTIN; depl_list_next++;
            depl_list[depl_list_next]=in;                   depl_list_next++;
            //if((rs232_in[in/8]&(1<<(in%8)))==0) wait=0;
        }


    }

    fclose(pf);

    // 3. verification des structures
    pf=fopen("fdcc_conf_seq_verif.txt","wt");
    if(pf==NULL) { printf("\nCannot open fdcc_conf_seq.txt\nBye\n\n"); exit(1); }
    fprintf(pf,"\n- DEPL_LIST (nb=%d)",depl_list_next);  for(i=0;i<DEPL_LIST_NB;i++) fprintf(pf,"\n%d %d %c",i,depl_list[i],depl_list[i]);
    fprintf(pf,"\n- DEPL"); for(i=0;i<DEPL_NB;i++) fprintf(pf,"\n%d %d",i,depl[i]);
    fprintf(pf,"\n- SEQ_LIST (nb=%d)",seq_list_next);  for(i=0;i<SEQ_LIST_NB;i++) fprintf(pf,"\n%d %d %c",i,seq_list[i],seq_list[i]);
    fprintf(pf,"\n- EXE"); for(i=0;i<EXE_NB;i++) { fprintf(pf,"\n%d: ",i); for(j=0;j<10;j++) fprintf(pf,"%d ",exe[i][j]); }
    fclose(pf);

    // 4. ouverture du port serie
    serial.init(sz_com,19200);
    for(i=0;i<100;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<100 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(4); serial.read(&data);  /*Sleep(10);*/ printf("\nPING:%x",data); // ping

    // 5. boucle
    cpt=0;
    step=0;

    // 5.1 execution du sequenceur
    int tempo_dcpt;
    int wait_end_run;
    int ec;
    pseq=0;
    tempo_dcpt=-1;
    wait_end_run=-1;
    while((ordre=seq_list[pseq])!=SEQ_ORDRE_FREE)
    {
        //printf("\nordre_seq=%d",ordre);
        txrx_masse();  // dure 250ms                 //printf(".%4d",step);
        cpt++;
        if(tempo_dcpt!=-1) tempo_dcpt--;

        printf("\n%d",cpt);

        if(ordre==SEQ_ORDRE_RUN)
        {
            for(e=0;e<EXE_NB;e++) if(exe[e][EXE_DEPL]==-1)
            {
                // si on est ici, au moins un canal d'execution est libre
                printf("\nSEQ LAUNCH DEPL%d on CH%d WAIT",exe[e][EXE_DEPL],e);
                exe[e][EXE_DEPL]=seq_list[pseq+1];
                exe[e][EXE_PTR]=depl[exe[e][EXE_DEPL]]+2; //saute instruction de reperage deplacement;
                //exe[e][EXE_LOCO]; // sera fait et verifie plus loin
                exe[e][EXE_WAIT]=-1;
                pseq+=2;
                break;
            }
        }

        if(ordre==SEQ_ORDRE_ENDRUN)
        {
            for(e=0;e<EXE_NB;e++) if(exe[e][EXE_DEPL]==seq_list[pseq+1]) ec=1;
            if(ec==0)
            {
                printf("\nSEQ ENDRUN DEPL%d ok",seq_list[pseq+1]);
                pseq+=2;
            }
        }

        if(ordre==SEQ_ORDRE_RUNWAIT)
        {
            if(wait_end_run==-1)
            {
                for(e=0;e<EXE_NB;e++) if(exe[e][EXE_DEPL]==-1)
                {
                    // si on est ici, au moins un canal d'execution est libre
                    exe[e][EXE_DEPL]=seq_list[pseq+1];
                    exe[e][EXE_PTR]=depl[exe[e][EXE_DEPL]]+2; //saute instruction de reperage deplacement;
                    //exe[e][EXE_LOCO]; // sera fait et verifie plus loin
                    exe[e][EXE_WAIT]=-1;
                    printf("\nSEQ LAUNCH DEPL%d on CH%d and WAIT",exe[e][EXE_DEPL],e);
                    wait_end_run=e;
                    break;
                }
            }
            else
            {
                if(exe[wait_end_run][EXE_DEPL]==-1) //remis a -1 en fin de deplacement
                {
                    printf("\nSEQ END OF DEPL%d on CH%d",exe[wait_end_run][EXE_DEPL],wait_end_run);
                    wait_end_run=-1;
                    pseq+=2;
                }
            }
        }

        if(ordre==SEQ_ORDRE_TEMPOS)
        {
            if(tempo_dcpt==-1){ tempo_dcpt=4*seq_list[pseq+1]; printf("\nSEQ tempos=%d",seq_list[pseq+1]); }
            if(tempo_dcpt==0) { tempo_dcpt=-1; pseq+=2; }
        }

        if(ordre==SEQ_ORDRE_SOUND)
        {
            i=0;
            while(seq_list[pseq+1+i]) { sz_sound[i]=(seq_list[pseq+1+i]); i++; }
            sz_sound[i]='\0';
            pseq+=(i+1+1);
            ShellExecute(NULL,"open",sz_sound, NULL, NULL, SW_SHOWNA);
            printf("\nSEQ sound %s",sz_sound);
        }

        if(ordre==SEQ_ORDRE_ECHO)
        {
            i=0;
            while(seq_list[pseq+1+i]) { sz_sound[i]=(seq_list[pseq+1+i]); i++; }
            sz_sound[i]='\0';
            pseq+=(i+1+1);
            printf("\nSEQ %s",sz_sound);
        }

        //5.2. execution des deplacements
        #define EPTR exe[e][EXE_PTR]
        for(e=0;e<EXE_NB;e++)
        {
            if(exe[e][EXE_DEPL]==-1) continue; // executeur inactif
            if(exe[e][EXE_WAIT]!=-1) exe[e][EXE_WAIT]--;

            while(1)
            {
                ordre=depl_list[EPTR];
                //pf=fopen("debug.txt","at");
                //fprintf(pf,"\nCH%d ordre=%d +1=%d +2=%d",e,depl_list[EPTR],depl_list[EPTR+1],depl_list[EPTR+2]);
                //fclose(pf);
                //printf("\nCH%d ordre=%d",e,ordre);

                if(ordre==DEPL_ORDRE_DEPL || ordre==DEPL_ORDRE_FREE)
                {
                    if(exe[e][EXE_WAIT]==-1)
                    {
                        exe[e][EXE_WAIT]=8; // 2sec
                        rs232_cmd[e]=0x60; //loco stop
                        rs232_fct[e]=0x90; //fct FL
                    }
                    if(exe[e][EXE_WAIT]==4)
                    {
                        rs232_adr[e]=0; //liberation de la locomotive
                    }
                    if(exe[e][EXE_WAIT]==0)
                    {
                        exe[e][EXE_DEPL]=-1;
                        printf("\nCH%d end",e);
                    }
                    break;
                }

                if(ordre==DEPL_ORDRE_LOCO)
                {
                    loco=depl_list[EPTR+1];
                    for(i=0;i<EXE_NB;i++) if(exe[i][EXE_LOCO]==loco) break;
                    // si on est ici, la locomotive n'est pas utilisee ailleur.
                    printf("\nCH%d loco=%d",e,loco);
                    rs232_cmd[e]=0x60;  //initialisation loco stop
                    rs232_fct[e]=0x90;  //initialisation fct FL
                    rs232_adr[e]=loco;  //prise de control de la locomotive
                    EPTR+=2;
                    continue;
                }

                if(ordre==DEPL_ORDRE_VIT)
                {
                    vit=depl_list[EPTR+2];
                    c=(char)depl_list[EPTR+1];
                    printf("\nCH%d vit= %c %d",e,c,vit);
                    cmd=(vit>>1)&0x0f;
                    if(vit&1) cmd|=0x10;
                    if(c=='+') cmd|=0x20;
                    cmd|=0x40;
                    rs232_cmd[e]=cmd;
                    EPTR+=3;
                    continue;
                }

                if(ordre==DEPL_ORDRE_AIG)
                {
                    aig=depl_list[EPTR+1];
                    c=(char)depl_list[EPTR+2];
                    if(c=='-') rs232_aig[aig/8]&=(0xff^(1<<(aig%8)));   //droit
                    if(c=='/') rs232_aig[aig/8]|=1<<(aig%8);            //devie
                    printf("\nCH%d aig(%d)=%c",e,aig,c);
                    EPTR+=3;
                    continue;
                }

                if(ordre==DEPL_ORDRE_FCT)
                {
                    fct=depl_list[EPTR+1];
                    val=depl_list[EPTR+2];
                    printf("\nCH%d fct(%d)=%d",e,fct,val);
                    msk=(1<<fct); if(msk&1) msk|=0x20; msk>>1;   //F0-F4.3.2.1
                    rs232_fct[e]&=(0xff^msk);
                    if(val) rs232_fct[e]|=msk;
                    EPTR+=3;
                    continue;
                }

                if(ordre==DEPL_ORDRE_WAITIN)
                {
                    in=depl_list[EPTR+1];
                    if(rs232_in[in/8]&(1<<(in%8)))
                    {
                        EPTR+=2;
                        //continue;
                        break; // pour plusieurs tests (pratique en capteur de courant en refoulement sur les voies de garage
                    }
                    break;
                }

                if(ordre==DEPL_ORDRE_WAITNOTIN)
                {
                    in=depl_list[EPTR+1];
                    if((rs232_in[in/8]&(1<<(in%8)))==0)
                    {
                        EPTR+=2;
                        //continue;
                        break;
                    }
                    break;
                }

                if(ordre==DEPL_ORDRE_TEMPOS)
                {
                    if(exe[e][EXE_WAIT]==-1) { exe[e][EXE_WAIT]=4*depl_list[EPTR+1]; printf("\nCH%d tempo=%d",e,exe[e][EXE_WAIT]); }
                    if(exe[e][EXE_WAIT]==0)  EPTR+=2;
                    break;
                }

                if(ordre==DEPL_ORDRE_TEMPOT)
                {
                    if(exe[e][EXE_WAIT]==-1) { exe[e][EXE_WAIT]=depl_list[EPTR+1]; printf("\nCH%d tempo=%d",e,exe[e][EXE_WAIT]); }
                    if(exe[e][EXE_WAIT]==0)  EPTR+=2;
                    break;
                }

                if(ordre==DEPL_ORDRE_SOUND)
                {
                    i=0;
                    while(depl_list[EPTR+1+i]) { sz_sound[i]=(depl_list[EPTR+1+i]); i++; }
                    sz_sound[i]='\0';
                    EPTR+=(i+1+1);
                    printf("\nCH%d SOUND:%s",e,sz_sound);
                    ShellExecute(NULL,"open",sz_sound, NULL, NULL, SW_SHOWNA);
                    continue;
                }

                if(ordre==DEPL_ORDRE_ECHO)
                {
                    i=0;
                    while(depl_list[EPTR+1+i]) { sz_sound[i]=(depl_list[EPTR+1+i]); i++; }
                    sz_sound[i]='\0';
                    EPTR+=(i+1+1);
                    printf("\nCH%d %s",e,sz_sound);
                    continue;
                }



            }  // fin d'execution d'un canal d'execution

        } // fin d'execution de tous les canaux auto

    } // fin d'execution du sequenceur


    // 6. femeture du port serie
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    serial.close();
}



/*****************************************************************************/
/* RESERVE: ANCIENNES FONCTIONS NON UTILISEEs, TESTS                         */
/*****************************************************************************/


// ancienne implementation pour un train a la fois
// permet de comprendre facilement l'implementation de la version courante
void main_auto_file_old(void)
{
    int i,j,data,data2,cpt,step,tempo;
    FILE* pf;
    int wait=0;
    char sz_ch[200];
    char sz_ordre[200];
    char sz_sound[500];
    int loco=0;
    int vit,val,cmd,fct,in,msk,aig,delay,start_ok=0;
    char c;

    serial.init(sz_com,19200);
    for(i=0;i<100;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<100 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(4); serial.read(&data);  /*Sleep(10);*/ printf("\nPING:%x",data); // ping

    cpt=0;
    step=0;
    txrx_masse();

    pf=fopen("fdcc_conf_seq.txt","rt");
    if(pf==NULL) { printf("\nCannot open fdcc_conf_seq.txt\nBye\n\n"); exit(1); }

    while(1)
    {
        if(fscanf(pf," %s",sz_ordre)==EOF) exit(1);

        if((sz_ordre[0]=='#') || (sz_ordre[0]=='/'))
        while(1)
        {
           if(fscanf(pf,"%c",&c)==EOF) break;
           if(c=='\n') break;
           if(c=='\r') break;
        }

        if(stricmp(sz_ordre,"start")==0) break;
    }


    while(1)
    {
        txrx_masse();  // dure 250ms
        printf(".%4d",step);
        cpt++;

        if(wait==0)
        {
            if(fscanf(pf," %s",sz_ordre)==EOF) break;
            printf("\nordre=%s",sz_ordre);
        }

        if((sz_ordre[0]=='#') || (sz_ordre[0]=='/'))
        while(1)
        {
           if(fscanf(pf,"%c",&c)==EOF) break;
           if(c=='\n') break;
           if(c=='\r') break;
        }

        if(stricmp(sz_ordre,"echo")==0)  // loco 0
        {
            while(1)
            {
                if(fscanf(pf,"%c",&c)==EOF) break;
                if(c=='\n') break;
                if(c=='\r') break;
                printf("%c",c);
            }
            printf("\n");
        }

        if(stricmp(sz_ordre,"end")==0) break;

        if(stricmp(sz_ordre,"stop")==0) break;

        if(stricmp(sz_ordre,"loco")==0)  // loco 0
        {
            if(fscanf(pf," %d",&loco)==EOF) break;
            printf(" loco=%d\n",loco);
        }

        if(stricmp(sz_ordre,"vit")==0)  // vit + 20
        {
            if(fscanf(pf," %c",&c)==EOF) break;

            if(fscanf(pf," %d",&vit)==EOF) break;
            printf(" vit= %c %d\n",c,vit);

            cmd=(vit>>1)&0x0f;
            if(vit&1) cmd|=0x10;
            if(c=='+') cmd|=0x20;
            cmd|=0x40;
            rs232_cmd[loco]=cmd;
        }

        if(stricmp(sz_ordre,"fct")==0)  // fct 0 1
        {
            if(fscanf(pf," %d",&fct)==EOF) break;

            if(fscanf(pf," %d",&val)==EOF) break;
            printf(" fct(%d)=%d\n",fct,val);

            msk=(1<<fct); if(msk&1) msk|=0x20; msk>>1;   //F0-F4.3.2.1
            rs232_fct[loco]&=(0xff^msk);
            if(val) rs232_fct[loco]|=msk;
        }

        if(stricmp(sz_ordre,"aig")==0)  // aig 3 -
        {
            if(fscanf(pf," %d",&aig)==EOF) break;

            if(fscanf(pf," %c",&c)==EOF) break;
            printf(" aig(%d)=%c\n",aig,c);

            if(c=='-') rs232_aig[aig/8]&=(0xff^(1<<(aig%8)));    //droit
            if(c=='/') rs232_aig[aig/8]|=1<<(aig%8);             //devie
        }

        if(stricmp(sz_ordre,"sound")==0)  // sound "c:\...."
        {
            fscanf(pf," %c",&c); // remove first \"
            i=0; sz_sound[0]='\0';
            while(1)
            {
                fscanf(pf,"%c",&c);
                if(c=='\"') break;
                sz_sound[i]=c; i++; sz_sound[i]='\0';
            }
            ShellExecute(NULL,"open",sz_sound, NULL, NULL, SW_SHOWNA);
        }

        if(stricmp(sz_ordre,"tempo")==0)  // tempo 40  (10s)
        {
            if(wait==0)
            {
            if(fscanf(pf," %d",&delay)==EOF) break;
            printf(" tempo=%d\n",delay);
            tempo=delay;
            wait=1;
            }

            if(tempo==0) wait=0;

            tempo--;
        }

        if(stricmp(sz_ordre,"tempos")==0)  // tempo 40  (10s)
        {
            if(wait==0)
            {
            if(fscanf(pf," %d",&delay)==EOF) break;
            printf(" tempos=%d\n",delay);
            tempo=delay*4;
            wait=1;
            }

            if(tempo==0) wait=0;

            tempo--;
        }

        if(stricmp(sz_ordre,"wait_in")==0)  // wait_in 8
        {
            if(wait==0)
            {
            if(fscanf(pf," %d",&in)==EOF) break;
            printf(" wait_in=%d\n",in);
            wait=1;
            }

            if(rs232_in[in/8]&(1<<(in%8))) wait=0;
        }

        if(stricmp(sz_ordre,"wait_not_in")==0)  // wait_in 8
        {
            if(wait==0)
            {
            if(fscanf(pf," %d",&in)==EOF) break;
            printf(" wait_not_in=%d\n",in);
            wait=1;
            }

            if((rs232_in[in/8]&(1<<(in%8)))==0) wait=0;
        }


    }

    fclose(pf);

    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    serial.close();
    //Sleep(10000);
    //loop: goto loop;

}



int table[8192];

int load_table(char* szFilename)
{
    FILE* pf;
    int i;
    unsigned int nb_word,adr,op,dath,datl,sum;
    char c;


    pf=fopen(szFilename,"rt");
    if(pf==NULL) { printf("\nCannot open file %s",szFilename); return 0; }

    printf("\nLoading %s",szFilename);

    for(i=0;i<8192;i++) table[i]=-1;

    while(1)
    {
        if(fscanf(pf," %c",&c)==EOF) break; //remove :
        fscanf(pf," %02x",&nb_word); nb_word=nb_word/2;
        fscanf(pf," %04x",&adr); adr=adr/2;
        fscanf(pf," %02x",&op);

        if(op==0) for(i=0;i<nb_word;i++)
        {
            fscanf(pf," %02x",&datl);
            fscanf(pf," %02x",&dath);
            table[adr+i]=(dath<<8)+datl;
        }

        fscanf(pf," %02x",&sum);
    }

    fclose(pf);

    for(i=0;i<8192;i++) if(table[i]!=-1) printf("\n%04x %04x",i,table[i]);

    return 1;
}



int main_kjkjkjj(int argc, char *argv[])
{
    int i,j,data;

    serial.init("COM2",19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    serial.write(8); Sleep(10); serial.write(0); Sleep(10); // set IRF pour bank0-1

    for(i=0;i<500;i++)
    {
        serial.write(2); Sleep(10); serial.write(0x2f); Sleep(10); // set ADR sur can0
        serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
        printf("\ncan0=%02x",data);
    }

    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    serial.close();
    loop: goto loop;

    return 0;
}


int main_iiiiuuu(int argc, char *argv[])
{
    int i,j,data;

    serial.init("COM2",19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    printf("\n\n");

    while(1)
    {
            serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2

            serial.write(2); Sleep(10); serial.write(0x29); Sleep(10); // set ADR
            serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            printf("\rir_data=%02x",data);

            serial.write(2); Sleep(10); serial.write(0x28); Sleep(10); // set ADR
            serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            printf(" ir_bit=%02x",data);

            serial.write(2); Sleep(10); serial.write(0x27); Sleep(10); // set ADR
            serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            printf(" ir_cpt=%02x",data);

            serial.write(2); Sleep(10); serial.write(0x2d); Sleep(10); // set ADR
            serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            printf(" ir_portc=%02x",data);

            serial.write(8); Sleep(10); serial.write(0); Sleep(10); // set IRF pour portc

            serial.write(2); Sleep(10); serial.write(0x07); Sleep(10); // set ADR
            serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
            printf(" portc=%02x",data);

    }
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    serial.close();
    //Sleep(10000);
   loop: goto loop;

    return 0;
}



int main___(int argc, char *argv[])
{
    int i,j,data;

    serial.init("COM2",19200);
    for(i=0;i<10;i++) { serial.write(0);    Sleep(10); } //sync
    for(i=0;i<2 ;i++) { serial.read(&data); Sleep(10); } //vide le buffer
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping
    printf("\n\n");

    while(1)
    {
            serial.write(8); Sleep(10); serial.write(1); Sleep(10); // set IRF pour bank2/3

            printf("\r");
            for(i=0;i<20;i++)
            {
                serial.write(2); Sleep(10); serial.write(0xd0+i); Sleep(10); // set ADR
                serial.write(4); Sleep(10); serial.read(&data);  Sleep(10); // read
                printf("%02x.",data);
            }
    }
    serial.write(1); Sleep(10); serial.read(&data);  Sleep(10); printf("\nPING:%x",data); // ping

    serial.close();
    //Sleep(10000);
   loop: goto loop;

    return 0;
}

